/**********************************************************
 ***********  support for Philips capture chip  ***********
 **********************************************************/

#include "vidcap.h"
#include "igs.h"
#include "igsvideo.h"

//extern unsigned char WhichCamera;
extern unsigned short wHighResThreshold;

/*for 7110 initialization */
#define P7110_Count 53
static unsigned char Phil7110_00[P7110_Count] =
{0x4c, 0x3f, 0x0d, 0xef, 0xbd, 0xf0, 0x00, 0x01,
 0xf8, 0xf8, 0x60, 0x60, 0x00, 0x86, 0x18, 0x70,
 0x00, 0x2c, 0x40, 0x50, 0x42, 0x1a, 0xfc, 0xd3,
 0xf0, 0x91, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 0xd9, 0x16, 0x40, 0x4c, 0x80, 0x40, 0x80, 0x41,
 0xfe, 0x01, 0xc5, 0x0f, 0x03, 0x01, 0x81, 0x03,
 0x44, 0x45, 0x02, 0x8c, 0x03
};

/*for 7111 initialization */
#define P7111_Count 17
static unsigned char Phil7111_02[P7111_Count] =
{								// 2    3    4    5    6    7    8    9
  0xc0,	 // c0=fuse, 38=gudl, 7=mode              <--auto gain, anti-alias
   0x23, // 40=hlnrs, 20=vbsl, 10=vpoff, 8=holdg, 4=gafix, 2=gai28, 1=gai18
   0x00, // gai10-17                              <--gain 0dB
   0x00, // gai20-27                              <--gain 0dB
   0xeb, // hsync start                           <-- -21 * 300ns = 6.2us
   0xe0, // hsync stop                            <-- -32 * 300ns = 9.5us
   0x48, // 80=aufd, 40=fsel, 20=exfil, 8=vtrc, 4=hpll, 3=vnoi <--ntsc,
   0x40, // 80=byps, 40=pref, 30=bpss, 8=vblb, 4=uptcv, 3=aper <--prefilter
	// a    b    c    d    e    f   10   11
   0x80, // luma brightness                       <-- CCIR default
   0x47, // luma contrast                         <-- 1.109 CCIR default
   0x40, // chroma saturation                     <-- CCIR default
   0x00, // chroma hue
   0x02, // 80=cdto, 40=cm99, 30=cstd, 8=dccf, 4=fctc, 3=chbw  <--medium BW, auto color
   0x00, // reserved
   0xc0, // c0=ofts, 30=hdel, 8=vrnl, 7=ydel      <--lumadelay=0, YUV656
   0x0c, // 80=gpsw, 20=feco, 10=compo, 8=oeyc, 4=oehv, 2=vipb, 1=colo
	//12
   0x80	//c0=rtse, 10=cbr, 8=rgb888, 4=dit, 3=aosl
};	/*reg_00 to reg_12 */




/**********************************************************
   Detect which decoder is hooked up
	 in: None
	 return PHILIPS7110 - Philips 7110 Decoder
	 PHILIPS7111 - Philips 7111 Decoder
	 NODECODER   - No decoder detected.
*********************************************************/
int
DetectDecoder (struct vidcap_size *pResult)
{
  int wCameraID = NODECODER;

#if 0
  if (sendOneByte (0x9C, 21, Phil7110_00[21]) == TRUE)
	{
	  wCameraID = PHILIPS7110;
//	  WhichCamera = wCameraID;
	  wHighResThreshold = 320 * 2;	/*as a indicator: Width < 320 or not */
	  pResult->width = 640;
	  pResult->height = 240;
	  return wCameraID;
	}
#endif

  outb_p (0xb6, IGSEXTINDEX);
  if (sendOneByte (P7111Port, 0x02, 0xD8) == TRUE)
	{
	  printk ("7111 Detected\n");
	  wCameraID = PHILIPS7111;
//	  WhichCamera = wCameraID;
	  wHighResThreshold = 352 * 2;	/*as a indicator: Width < 352 or not */
	  pResult->width = 704;
	  pResult->height = 240;
	  return wCameraID;
	}
  else
	printk ("7111 NOT Detected\n");

  return wCameraID;

}

/**********************************************************
 Initialize Philips 7111
 in:   none
 return  TRUE  if successful
	 FALSE if error
 *********************************************************/
int
InitPhil7111 (void)
{
  if (sendMoreByte (P7111Port, 0x02, Phil7111_02, P7111_Count) == FALSE)
	return FALSE;

  return TRUE;
}


/**
 * InteGraphics Systems code:
 * Communication routines to talk to the Philips SAA7111 via the CyberPro2000
 * registers 3ce (index:b6) and 3cf (data).
 * Communication between CyberPro2000 and SAA7111 is accomplished by using the
 * I2C interface.
 * IMPORTANT: I2C line is LO.
 * [Slave addresses change according to the status of the I2C line (HI/LO)]
 *
 * TODO:
 *	Change the delay() function to udelay().
 *  Check if udelay can be used in kernel code.
 */

/*=========================================================
 This routine is used internally
 in:   bVal
 out:  return ture if successful
 =========================================================*/
int
writeI2C (unsigned char bVal)
{
  int i;
  unsigned char bBit = 0x80;	/* Start from high bit -- Bit 7. Write each bit. */

  int bReturn = TRUE;

  for (i = 0; i < 8; i++)
	{
	  if (bVal & bBit)
		{
		  outb_p (0xb6, IGSEXTINDEX);
		  outb_p (0x40, IGSEXTDATA);
		  udelay (TIME1);
		  outb_p (0x50, IGSEXTDATA);
		  udelay (TIME1);

		  if (AcknowledgeClk () != 0)
			bReturn = FALSE;

		  outb_p (0x40, IGSEXTDATA);
		  udelay (TIME1);
		}
	  else
		{
		  outb_p (0xb6, IGSEXTINDEX);
		  outb_p (0x00, IGSEXTDATA);
		  udelay (TIME1);
		  outb_p (0x10, IGSEXTDATA);
		  udelay (TIME1);

		  if (AcknowledgeClk () != 0)
			bReturn = FALSE;

		  outb_p (0x00, IGSEXTDATA);
		  udelay (TIME1);
		}
	  bBit = bBit >> 1;
	}

  return bReturn;
}


/*=========================================================
 This routine is used internally
 in:   none
 return  0 if successful
	-1 if error
 =========================================================*/
int
Acknowledge (void)
{
  unsigned long i = 0L;

  outb_p (0xb6, IGSEXTINDEX);
  outb_p (0x40, IGSEXTDATA);
  udelay (TIME1);
  outb_p (0x50, IGSEXTDATA);
  udelay (TIME1);
  while ((inb_p (IGSEXTDATA) & 0x80) && (i++ < TimeDelay1));

  outb_p (0x40, IGSEXTDATA);
  udelay (TIME1);
  outb_p (0x00, IGSEXTDATA);
  udelay (TIME1);

  if (i < TimeDelay1)
	return 0;
  else
	return -1;
}


/*=========================================================
 This routine is used internally
 in:   none
 return  0 if successful
	-1 if error
 =========================================================*/
int
AcknowledgeClk (void)
{
  unsigned long i = 0;

  outb_p (0xb6, IGSEXTINDEX);
  while (!(inb_p (IGSEXTDATA) & 0x20) && (i++ < TimeDelay1));

  if (i < TimeDelay1)
	return 0;
  else
	return -1;
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
int
Start_Cycle (void)
{
  int bReturn = TRUE;

  outb_p (0xb6, IGSEXTINDEX);
  udelay (TIME1);
  outb_p (0x50, IGSEXTDATA);
  udelay (TIME1);
  outb_p (0x10, IGSEXTDATA);
  udelay (TIME1);

  if (AcknowledgeClk () != 0)
	bReturn = FALSE;

  udelay (TIME1);
  outb_p (0x00, IGSEXTDATA);
  udelay (TIME1);

  return bReturn;
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
int
Stop_Cycle (void)
{
  int bReturn = TRUE;

  outb_p (0xb6, IGSEXTINDEX);
  udelay (TIME1);
  outb_p (0x10, IGSEXTDATA);
  udelay (TIME1);
  outb_p (0x50, IGSEXTDATA);
  udelay (TIME1);

  if (AcknowledgeClk () != 0)
	bReturn = FALSE;

  return bReturn;
}


/**********************************************************
Program one byte value to I2C of the Decoder
 in:	 bAddr	  = I2C port address
	 bSubaddr = I2C subport address
	 bVal	  = date to be programmed
 return  TRUE  if successful
	 FALSE if error
 *********************************************************/
int
sendOneByte (unsigned char bAddr, unsigned char bSubaddr, unsigned char bVal)
{
  int bReturn = TRUE;

  if (Start_Cycle () == FALSE)
	bReturn = FALSE;

  if (writeI2C (bAddr) == FALSE)
	bReturn = FALSE;

  if (Acknowledge () != 0)
	bReturn = FALSE;

  if (writeI2C (bSubaddr) == FALSE)
	bReturn = FALSE;

  if (Acknowledge () != 0)
	bReturn = FALSE;

  if (writeI2C (bVal) == FALSE)
	bReturn = FALSE;

  if (Acknowledge () != 0)
	bReturn = FALSE;

  if (Stop_Cycle () == FALSE)
	bReturn = FALSE;

  udelay (STOPTIME);

  return bReturn;
}


/**********************************************************
Program more bytes to I2C of the Decoder
 in:	 bAddr	  = I2C port address
	 bSubaddr = I2C subport address
	 bValPtr  = pointer to data array
	 iCount   = counter (# of bytes to be programmed)
 return  TRUE  if successful
	 FALSE if error
 *********************************************************/
int
sendMoreByte (unsigned char bAddr, unsigned char bSubaddr, unsigned char *bValPtr, int iCount)
{
  int bReturn = TRUE;
  unsigned char bVal;
  int i;

  if (Start_Cycle () == FALSE)
	bReturn = FALSE;

  if (writeI2C (bAddr) == FALSE)
	bReturn = FALSE;

  if (Acknowledge () != 0)
	bReturn = FALSE;

  if (writeI2C (bSubaddr) == FALSE)
	bReturn = FALSE;

  if (Acknowledge () != 0)
	bReturn = FALSE;

  for (i = 0; i < iCount; i++)
	{
	  bVal = *bValPtr++;
	  if (writeI2C (bVal) == FALSE)
		bReturn = FALSE;

	  if (Acknowledge () != 0)
		bReturn = FALSE;
	}

  if (Stop_Cycle () == FALSE)
	bReturn = FALSE;

  udelay (STOPTIME);

  return bReturn;
}
